//------------------------------------------------------------------------------
//
// Copyright (C) 2006, Xilinx, Inc. All Rights Reserved.
//
// This file is owned and controlled by Xilinx and must be used solely
// for design, simulation, implementation and creation of design files
// limited to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and immediately
// terminates your license.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor      : Xilinx
// \   \   \/     Version     : 1.5
//  \   \         Application : Generated by Xilinx PCI Express Wizard
//  /   /         Filename    : pcie_clocking.v
// /___/   /\     Module      : pcie_clocking 
// \   \  /  \
//  \___\/\___\
//
//------------------------------------------------------------------------------

`timescale 1 ns / 10 ps


module pcie_clocking #
  ( parameter    G_USE_DCM     = 1,
    parameter    G_DIVIDE_VAL  = 2,  // use 2 or 4
    parameter    CLKFBOUT_MULT = 5,
    parameter    CLKIN_PERIOD  = 10
  )
  (
    input       clkin_pll,
    input       clkin_dcm,
    input       rst,
    
    output      coreclk,
    output      userclk,
    output      locked
  );
 

    wire        clkfbout;
    wire        clkfbin;
    wire        clkout0;
    wire        clkout1;
    
    wire        clk0;
    wire        clkfb_bufg;
    wire        clkdv;
    wire [6:0]  not_connected;
    

    parameter G_DIVIDED_VAL_PLL = G_DIVIDE_VAL*2;
   
// DCM Section

    generate
       if (G_USE_DCM == 1 && G_DIVIDE_VAL == 1) 
        begin : use_bufg
            assign coreclk = clkin_dcm;
            assign userclk = clkin_dcm;
            assign locked = 1'b1;
        end
    endgenerate

    generate
    if (G_USE_DCM == 1 && G_DIVIDE_VAL > 1) 
        begin : use_dcm
   
   
    // Instantiate a DCM module to divide the reference clock.
        DCM_BASE #
        (
           .CLKDV_DIVIDE               (G_DIVIDE_VAL),
           .CLKIN_PERIOD               (CLKIN_PERIOD),
           .DFS_FREQUENCY_MODE         ("LOW"), 
           .DLL_FREQUENCY_MODE         ("HIGH")
        )
        dcm_i
        (
           .CLK0                       (clk0),
           .CLK180                     (not_connected[0]),
           .CLK270                     (not_connected[1]),
           .CLK2X                      (not_connected[2]),
           .CLK2X180                   (not_connected[3]),
           .CLK90                      (not_connected[4]),
           .CLKDV                      (clkdv),
           .CLKFX                      (not_connected[5]),
           .CLKFX180                   (not_connected[6]),
           .LOCKED                     (locked),
           .CLKFB                      (clkfb_bufg),
           .CLKIN                      (clkin_dcm),
           .RST                        (rst)
        );
        
        BUFG clkfb_dcm_bufg  (.O(clkfb_bufg),.I(clk0));   // 250 MHz
        BUFG usrclk_dcm_bufg (.O(userclk),.I(clkdv));     // 125 MHz or 62.5 Mhz
        
        assign coreclk = clkfb_bufg;
        
        end
    endgenerate



// PLL Section

    generate
    if (G_USE_DCM == 0)     
        begin : use_pll

         // OR CALCULATED WAY (easier): must use bitgen -g plladv_xNyM_use_calc:Yes to enable this attribute
        // synthesis attribute CLKOUT1_DIVIDE pll1 "4";
        // synthesis attribute CLKOUT1_PHASE pll1 "0";
        // synthesis attribute CLKOUT1_DUTY_CYCLE pll1 "0.5";

        // synthesis attribute PLL_OPT_INV pll1 "001001";
        // synthesis attribute PLL_CP pll1 "2";
        // synthesis attribute PLL_RES pll1 "8";

           PLL_BASE #
           (
                .CLKFBOUT_MULT (CLKFBOUT_MULT),
                .CLKFBOUT_PHASE(0),
                
                .CLKIN_PERIOD (CLKIN_PERIOD),
                
                .CLKOUT0_DIVIDE(2), // always 250 MHz for core_clk
                .CLKOUT0_PHASE (0),
                
                .CLKOUT1_DIVIDE(G_DIVIDED_VAL_PLL),
                .CLKOUT1_PHASE (0)                
           )
           pll_i
           
           (
            .CLKIN(clkin_pll),
            .CLKFBIN(clkfbin),
            .RST(rst),
            .CLKOUT0(clkout0),
            .CLKOUT1(clkout1),
            .CLKOUT2(not_connected[0]),
            .CLKOUT3(not_connected[1]),
            .CLKOUT4(not_connected[2]),
            .CLKOUT5(not_connected[3]),
            .CLKFBOUT(clkfbout),
            .LOCKED(locked)
           );

            BUFG clkfbin_pll_bufg  (.O(clkfbin),    .I(clkfbout));
            BUFG coreclk_pll_bufg  (.O(coreclk),    .I(clkout0));  // 250 MHz


        if (G_DIVIDE_VAL == 1) begin: userclk_250
            assign userclk = coreclk;
          
        end else begin: not_250
           
           BUFG usrclk_pll_bufg   (.O(userclk),    .I(clkout1));
        end
        
    end
    endgenerate      

endmodule
