#!xilperl

##-----------------------------------------------------------------------------
##
## Copyright (C) 2007, Xilinx, Inc. All Rights Reserved.
##
## This file is owned and controlled by Xilinx and must be used solely
## for design, simulation, implementation and creation of design files
## limited to Xilinx devices or technologies. Use with non-Xilinx
## devices or technologies is expressly prohibited and immediately
## terminates your license.
##
## Xilinx products are not intended for use in life support
## appliances, devices, or systems. Use in such applications is
## expressly prohibited.
##
##   ____  ____
##  /   /\/   /
## /___/  \  /    Vendor      : Xilinx
## \   \   \/     Version     : 1.0
##  \   \         Application : XAPP869 
##  /   /         				
## /___/   /\     Filename    : make_connectivity.pl
## \   \  /  \
##  \___\/\___\
##
##-----------------------------------------------------------------------------
##
##	Description : This is a Perl script to generate the required
##					  design with given parameters for the reference design (XAPP869)
##	"Point-to-point Connectivity Using Integrated Endpoint Block for PCI Express Designs"

##	NOTES	:	Before running this script, set up your xilinx tools environment.
##			This script uses 'xilperl' a version of PERL interpreter shipped with
##				Xilinx tools.

##---Check for the required environment variable---###

unless($xilinx_path = $ENV{XILINX})
    {
            print" XILINX environment variable has not been set.  This variable\n";
            print" points to your Xilinx ISE tools, and is required to run script for\n";
            print" this reference design. Set up the Xilinx tools environment before running\n";
            print" this script.\n";
            exit;
    }
    
##---Set default values---##

my $device = "xc5vlx110t-1-ff1136";	##--Set Device
my $design_option = 1;					##--User option 
my $lanes = 4;								##--Number of lanes set to 4 by default
my $use_example = 0;	##--Example design bitfile generation is false by default
my $speed = -1;
my $board = "ML523";
my $SI_version = "PS";	##---Device assumed Production by default

##---Extract Command Line Parameters---##

while ($option = shift @ARGV)
{
	if ($option =~ /^-device/)				##--device option
	{
		@cli_option = split /=/, $option;
		$device = $cli_option[1];
	}
	if ($option =~ /^-design_option/)	##--design option
	{
		@cli_option = split /=/, $option;
		$design_option = $cli_option[1];
	}
	if ($option =~ /^-board/)				##--target board option
	{
		@cli_option = split /=/, $option;
		$board = uc($cli_option[1]);
	}
	if ($option =~ /^-lanes/)				##--lane configuration option
	{
		@cli_option = split /=/, $option;
		$lanes = $cli_option[1];
		if (($lanes != 1) && ($lanes != 2) && ($lanes != 4) && ($lanes != 8)) {
			print "Invalid value entered for lanes\n";
			exit;
		}
	}
	if ($option =~ /^-example/i)	##--example design switch	
	{
		$use_example = 1;
	}
	if ($option =~ /^-es/i)			##--engineering sample silicon switch
	{
		$SI_version = "ES";
	}
	if ($option =~ /^-ps/i)			##--production sample silicon switch
	{
		$SI_version = "PS";
	}
	if (($option =~ /^-h/) || ($option =~ /^-help/))##--help on script parameters
	{
		show_help();
		exit;
	}
}	##---command line parameter extraction completed---##

	##--Only x1 designs allowed on ML505--##
if ($board eq "ML505" && $lanes != 1)
{
	print "Setting the number of lanes to 1\n";
	$lanes = 1;
}
	##--Device part fixed on ML505--##
if ($board eq "ML505")
{
	print "Setting device to XC5VLX50T for ML505 board\n";
	$device = "xc5vlx50t-1-ff1136";
}

	##--Design option selection--##
if ($design_option > 2 || $design_option < 1) {
	print "Design option out of range - setting it to default value of 1.\n";
	$design_option = 1;
}

	##--extract speed of device from device name--##
@dev_list = split (/-/,$device);
	if ($dev_list[1] > 3 && $dev_list[1] < 1) {
		$speed = -1;
	} else {
		$speed = -$dev_list[1];
	}
	##--Set UCF name--##
if ($use_example) {
	$ucf_file_name = "../../ucf/connectivity_x".$lanes."_".$board."_"."$SI_version".".ucf";
}

##--make a list of files required for synthesis--##

my @MASTER_MODULES = (
	"../src/config_fsm.v",
	"../src/connectivity_master_top.v",
	"../src/pcie_master.v",
	"../src/pcie_master_top.v"
);		##--end master module listing

my @SLAVE_MODULES = (
	"../src/link_rdy_us.v",
	"../src/connectivity_slave_top.v",
	"../src/pcie_slave.v",
	"../src/pcie_slave_top.v"
);		##--end slave module listing

 ##--Change gt_wrapper location here if different device ES version is used--##
if ($board eq "ML505")
{
	$gt_wrapper = "../pcie_wrapper_files/pcie_gt_wrapper_x1_ML505"."_"."$SI_version".".v";
} else {
	$gt_wrapper = "../pcie_wrapper_files/pcie_gt_wrapper_x".$lanes."_"."$SI_version".".v";
}

	##--Wrapper modules for Integrated Endpoint block--##
my @PCIE_WRAPPER_MODULES = (
	"../pcie_wrapper_files/pcie_clocking.v",
	"../pcie_wrapper_files/pcie_mim_wrapper.v",
	"../pcie_wrapper_files/pcie_reset_logic.v",
	"../pcie_wrapper_files/bram_common.v",
	$gt_wrapper
	);

	##--Modules common to both master & slave designs--##
my @SRC_MODULES = (
	@PCIE_WRAPPER_MODULES,
	"../src/context_ip_tx.v",
	"../src/len_counter.v",
	"../src/frame_error_detect.v",
	"../src/tx_control_gen.v",
	"../src/requester_logic.v",
	"../src/tx_data_validate_fsm.v",
	"../src/down_counter.v",
	"../src/tli_control_fsm.v",
	"../src/context_ip_rx.v",
	"../src/rx_read_tlp_fsm.v",
	"../src/rx_frame_ll_fsm.v"
);		##--end common source module listing

use Cwd;
$working_dir = cwd;

##--Generate Simulation DO file--##
	##--Simulation directory--##
$sim_dir = "../simulation";
open DO_FILE, ">$sim_dir/sim.do";
write_do_file(0,$lanes);
close DO_FILE;
system ("chmod +755 $sim_dir/sim.do");
print "##--DO file for simulation ready--##\n";

##--Generate Top level files--##
$ex_dir = "../example_design";

open TOP_FILE, ">$ex_dir/connectivity_master.v";
write_top_file(1,$lanes,$design_option);
close TOP_FILE;
system ("chmod +755 $ex_dir/connectivity_master.v");
print "##--file connectivity_master.v generated--##\n";

open EX_FILE, ">$ex_dir/connectivity_sample_master.v";
write_sample_file(1,$lanes,$design_option);
close EX_FILE;
system ("chmod +755 $ex_dir/connectivity_sample_master.v");
print "##--file connectivity_sample_master.v generated--##\n";

open TOP_FILE, ">$ex_dir/connectivity_slave.v";
write_top_file(0,$lanes,$design_option);
close TOP_FILE;
system ("chmod +755 $ex_dir/connectivity_slave.v");
print "##--file connectivity_slave.v generated--##\n";

open EX_FILE, ">$ex_dir/connectivity_sample_slave.v";
write_sample_file(0,$lanes,$design_option);
system ("chmod +755 $ex_dir/connectivity_sample_slave.v");
close EX_FILE;
print "##--file connectivity_sample_slave.v generated--##\n";

## -- This generates the post-synthesis netlist for functional simulation 
## 			when example design bitstream is to be generated --##
if ($use_example) {	#--generate simulation netlist using top level wrapper
##---Modules for generating simulation netlist---##
	@SIM_MASTER =  (	@MASTER_MODULES,
							@SRC_MODULES,
							"../example_design/connectivity_master.v"
						);

	@SIM_SLAVE =   (  @SLAVE_MODULES,
                     @SRC_MODULES,
                     "../example_design/connectivity_slave.v"
                  );
	
	if (-d "master")
	{
   	system ("rm -rf master");
	}
   system ("mkdir master");
	$master_dir = $working_dir."/master";

	##----Generate scripts for Slave----##
	chdir $master_dir;
	##--Create XST script file for Master--##
	open SCR_FILE, ">make_master.scr";
	write_scr_file (1,0,$device);
	close SCR_FILE;
	system ("chmod +755 $master_dir/make_master.scr");

	open PRJ_FILE, ">make_master.prj";
	write_prj_file(@SIM_MASTER);
	close PRJ_FILE;
	system ("chmod +755 $master_dir/make_master.prj");

	system ("xst -ifn make_master.scr -intstyle silent");
	if (! -e "connectivity_master.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	print "Generating Functional Simulation netlist for master\n";
	system ("netgen -w -sim -ofmt verilog -intstyle silent -dir ../../simulation connectivity_master.ngc");

	chdir $working_dir;

	if (-d "slave")
	{
   	system ("rm -rf slave");
	}
   	system ("mkdir slave");

	$slave_dir = $working_dir."/slave";
	chdir $slave_dir;

	open SCR_FILE, ">make_slave.scr";
	write_scr_file (0,0,$device);
	close SCR_FILE;
	system ("chmod +755 $slave_dir/make_slave.scr");

	print "Generating XST Project File for Slave\n";
	open PRJ_FILE, ">make_slave.prj";
	write_prj_file(@SIM_SLAVE);
	close PRJ_FILE;
	system ("chmod +755 $slave_dir/make_slave.prj");

	system ("xst -ifn make_slave.scr -intstyle silent");
	if (! -e "connectivity_slave.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	print "Generating Functional Simulation netlist for slave\n";
	system ("netgen -w -sim -ofmt verilog -intstyle silent -dir ../../simulation connectivity_slave.ngc");
	chdir $working_dir;
}	##-----simulation netlist generation ends----##

	##---Module grouping for synthesis---##
if ($use_example)
{
	@MASTER_MODULES = ( @MASTER_MODULES,
						@SRC_MODULES,
						"../example_design/frame_gen.v",
						"../example_design/frame_check.v",
						"../example_design/connectivity_master.v",
						"../example_design/connectivity_sample_master.v"
					  );
					   
	@SLAVE_MODULES = ( @SLAVE_MODULES,
					   @SRC_MODULES,
					   "../example_design/frame_gen.v",
					   "../example_design/frame_check.v",
					   "../example_design/connectivity_slave.v",
					   "../example_design/connectivity_sample_slave.v"
					 );
} else {
	@MASTER_MODULES = ( @MASTER_MODULES,
							  @SRC_MODULES,
							  "../example_design/connectivity_master.v"
							);

	@SLAVE_MODULES = ( @SLAVE_MODULES,
							 @SRC_MODULES,
					   	 "../example_design/connectivity_slave.v"
						  );	
}

##--Make folder for synthesis & implementation (in case of example design)--##
if (-d "master")
{
	system ("rm -rf master");
}
	system ("mkdir master");

if (-d "slave")
{
	system ("rm -rf slave");
}
	system ("mkdir slave");

$master_dir = $working_dir."/master";
$slave_dir = $working_dir."/slave";

##----Generate scripts for Slave----##
chdir $master_dir;
##--Create XST script file for Master--##
open SCR_FILE, ">make_master.scr";
write_scr_file (1,$use_example,$device);
close SCR_FILE;
system ("chmod +755 $master_dir/make_master.scr");

##--Create XST project file for Master--##
print "Generating XST Project File for Master\n";
open PRJ_FILE, ">make_master.prj";
write_prj_file(@MASTER_MODULES);
close PRJ_FILE;
system ("chmod +755 $master_dir/make_master.prj");

print "Generating NGC file (netlist) for master design\n";
system ("xst -ifn make_master.scr -intstyle silent");

if ($use_example == 0) {
	if (! -e "connectivity_master.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	print "Generating Functional Simulation netlist from NGC for master\n";
	system ("netgen -w -sim -ofmt verilog -intstyle silent -dir ../../simulation connectivity_master.ngc");
}
	##--Generate bitstream for example design--##
if ($use_example) {
	if (! -e "connectivity_sample_master.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	system("cp ../../chipscope_files/*.edn .");
	system ("ngdbuild -sd . -uc $ucf_file_name -p $device connectivity_sample_master -intstyle silent");
	if (! -e "connectivity_sample_master.ngd") {
		print ("Error during NGD build! NGD file not created.\n");
		exit;
	}
	print "##----Run MAP----##\n";
	if (($lanes == 8) && ($design_option == 2)) {
		system ("map -timing -p $device -ol med -o connectivity_sample_master_map.ncd connectivity_sample_master.ngd mapped.pcf -intstyle silent");
	} else {
		system ("map -timing -p $device -ol high -o connectivity_sample_master_map.ncd connectivity_sample_master.ngd mapped.pcf -intstyle silent");
	}
	if (! -e "connectivity_sample_master_map.ncd") {
		print ("Error during MAP! NCD file not created.\n");
		exit;
	}
	print "##----MAP DONE----##\n";
	
	print "##----Run PAR----##\n";
	system ("par -ol high -w connectivity_sample_master_map.ncd connectivity_sample_master_routed.ncd mapped.pcf -intstyle silent");
	if (! -e "connectivity_sample_master_routed.ncd") {
		print ("Error during PAR! NCD file not created.\n");
		exit;
	}
	print "##----PAR DONE----##\n";
	
	print "##----Run TRCE----##\n";
	system ("trce -u -e 10 -s $speed connectivity_sample_master_routed -o connectivity_sample_master mapped.pcf");
	if (! -e "connectivity_sample_master.twr") {
		print ("Error during TRCE! TWR file not created.\n");
		exit;
	}
	print "##----TRCE DONE----##\n";

	print "##----Run BITGEN----##\n";
	system ("bitgen -dw -g plladv_x0y2_use_calc:Yes -g StartupClk:Cclk connectivity_sample_master_routed connectivity_sample_master");
	print "##----BITGEN DONE----##\n";
	print "##---Master bitfile connectivity_sample_master.bit generated---##\n";
}

##----Generate scripts for Slave----##
chdir $slave_dir;

##--Create XST script file for Slave--##
open SCR_FILE, ">make_slave.scr";
write_scr_file (0,$use_example,$device);
close SCR_FILE;
system ("chmod +755 $slave_dir/make_slave.scr");

##--Create XST project file for Slave--##
print "Generating XST Project File for Slave\n";
open PRJ_FILE, ">make_slave.prj";
write_prj_file(@SLAVE_MODULES);
close PRJ_FILE;
system ("chmod +755 $slave_dir/make_slave.prj");

print "Generating NGC file (netlist) for Slave design\n";
system ("xst -ifn make_slave.scr -intstyle silent");

if ($use_example == 0) {
	if (! -e "connectivity_slave.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	print "Generating Functional Simulation netlist from NGC for slave\n";
	system ("netgen -w -sim -ofmt verilog -intstyle silent -dir ../../simulation connectivity_slave.ngc");
}
	##--Generate bitstream for example design--##
if ($use_example) {
	if (! -e "connectivity_sample_slave.ngc") {
		print ("Error during synthesis! NGC file not created.\n");
		exit;
	}
	system("cp ../../chipscope_files/*.edn .");
	print "##----Run NGDBuild----##\n";	
	system ("ngdbuild -sd . -uc $ucf_file_name -p $device connectivity_sample_slave -intstyle silent");
	if (! -e "connectivity_sample_slave.ngd") {
		print ("Error during NGD build! NGD file not created.\n");
		exit;
	}
	print "##----NGDBuild DONE----##\n";
	
	print "##----Run MAP----##\n";
	if (($lanes == 8) && ($design_option == 2)) {
		system ("map -timing -p $device -ol med -o connectivity_sample_slave_map.ncd connectivity_sample_slave.ngd mapped.pcf -intstyle silent");
	} else {
		system ("map -timing -p $device -ol high -o connectivity_sample_slave_map.ncd connectivity_sample_slave.ngd mapped.pcf -intstyle silent");
	}
	if (! -e "connectivity_sample_slave_map.ncd") {
		print ("Error during MAP! NCD file not created.\n");
		exit;
	}
	print "##----MAP DONE----##\n";
	
	print "##----Run PAR----##\n";
	system ("par -ol high -w connectivity_sample_slave_map.ncd connectivity_sample_slave_routed.ncd mapped.pcf -intstyle silent");
	if (! -e "connectivity_sample_slave_routed.ncd") {
		print ("Error during PAR! NCD file not created.\n");
		exit;
	}
	print "##----PAR DONE----##\n";
	
	print "##----Run TRCE----##\n";
	system ("trce -u -e 10 -s $speed connectivity_sample_slave_routed -o connectivity_sample_slave mapped.pcf");
	if (! -e "connectivity_sample_slave.twr") {
		print ("Error during TRCE! TWR file not created.\n");
		exit;
	}
	print "##----TRCE DONE----##\n";
	
	print "##----Run BITGEN----##\n";
	system ("bitgen -dw -g plladv_x0y2_use_calc:Yes -g StartupClk:Cclk connectivity_sample_slave_routed connectivity_sample_slave");
	print "##----BITGEN DONE----##\n";
	print "##---Slave bitfile connectivity_sample_slave.bit generated---##\n";
}

chdir $working_dir;

##------------Subroutine for help------------------##
sub show_help
{
	print "\n-----make_connectivity HELP-----\n";	
	print "make_connectivity [options]\n";
	print "\t -lanes=<number_of_lanes> \t Give number of lanes for design 1/2/4/8 \n";
	print "\t -design_option=<1/2> \t\t Give design option 3 Block RAM with \n\t\t\t1.62Gbps/lane or 6 Block RAM with 1.74Gbps/lane\n";
	print "\t -board=<target_board> \t\t Enter target board - ML505/ML523\n";
	print "\t -example \t\t\t Generates bitstream for example design\n";
	print "\t -device=<target_device> \t Enter full device name like \n\t\t\t\t\t xc5vlx50t-1-ff1136\n";
	print "\t -es/ps \t\t\t Select Silicon sample \n\t\t\t\t\t ps for Production part &\n\t\t\t\t\t es for Engineering sample\n";
	print "\t -h \t\t\t\t Displays this message\n\n";
}

##------------Subroutine for writing DO file------------------##
sub write_do_file
{
	my ($example,$lanes) = @_;

	print DO_FILE "set XILINX ", $ENV{XILINX},"\n\n";
	print DO_FILE "vlib work\n";
	print DO_FILE "vmap work work\n\n";
	print DO_FILE "## Compile HDL files\n\n";
	print DO_FILE "vlog -work work ../example_design/frame_gen.v;\n";	
	print DO_FILE "vlog -work work ../example_design/frame_check.v;\n";	
	print DO_FILE "vlog -work work connectivity_master.v;\n";	
	print DO_FILE "vlog -work work ../example_design/connectivity_sample_master.v;\n";	
	print DO_FILE "vlog -work work connectivity_slave.v;\n";	
	print DO_FILE "vlog -work work ../example_design/connectivity_sample_slave.v;\n";	
	print DO_FILE "vlog -work work connectivity_sample_tb.v;\n";	
	print DO_FILE "vlog -work work \$XILINX/verilog/src/glbl.v\n";	
	print DO_FILE "\n## Simulation command\n\n";

	print DO_FILE "vsim -t 1ps -L UNISIMS_VER -GNO_OF_LANES=$lanes work.connectivity_sample_tb work.glbl\n";
	print DO_FILE "view wave\n";
	print DO_FILE "do wave.do\n";
	
	print DO_FILE "run 75us\n";
}

##------------Subroutine for writing EXAMPLE file------------------##
sub write_sample_file
{
	my ($master,$lanes,$design_option) = @_;
	my $lanes_1 = $lanes - 1;
		##--print file header--##

	print EX_FILE "//------------------------------------------------------------------------------\n";
   print EX_FILE "//\n";
	print EX_FILE "// Copyright (C) 2007, Xilinx, Inc. All Rights Reserved.\n";
	print EX_FILE "//\n";
	print EX_FILE "// This file is owned and controlled by Xilinx and must be used solely\n";
	print EX_FILE "// for design, simulation, implementation and creation of design files\n";
	print EX_FILE "// limited to Xilinx devices or technologies. Use with non-Xilinx\n";
	print EX_FILE "// devices or technologies is expressly prohibited and immediately\n";
	print EX_FILE "// terminates your license.\n";
	print EX_FILE "//\n";
	print EX_FILE "// Xilinx products are not intended for use in life support\n";
	print EX_FILE "// appliances, devices, or systems. Use in such applications is\n";
	print EX_FILE "// expressly prohibited.\n";
	print EX_FILE "//\n";
	print EX_FILE "//   ____  ____\n";
	print EX_FILE "//  /   /\\/   /\n";
	print EX_FILE "// /___/  \\  /    Vendor      : Xilinx\n";
	print EX_FILE "// \\   \\   \\/     Version     : 1.0\n";
	print EX_FILE "//  \\   \\         Application : XAPP869\n";
	if ($master){
	print EX_FILE "//  /   /         Filename    : connectivity_sample_master.v\n"
	}else{
	print EX_FILE "//  /   /         Filename    : connectivity_sample_slave.v\n"
	}
	print EX_FILE "// /___/   /\\\n";
	print EX_FILE "// \\   \\  /  \\\n";
	print EX_FILE "//  \\___\\/\\___\\\n";
	print EX_FILE "//\n";
	print EX_FILE "//------------------------------------------------------------------------------\n";
	if ($master) {
		print EX_FILE "module connectivity_sample_master (\n";
	} else {
		print EX_FILE "module connectivity_sample_slave (\n";
	}
	print EX_FILE "\t input 	clk_n,\n";
	print EX_FILE "\t input 	clk_p,\n";
	print EX_FILE "\t input 	rst_n,\n";
	print EX_FILE "\t input 	reset_init,\n";
	print EX_FILE "\t input 	gtpreset_n,\n";
	print EX_FILE "\t output 	link_ready,\n";
	print EX_FILE "\t output 	linkup,\n";
	print EX_FILE "\t output [3:0] link_width,\n";
	print EX_FILE "\t output 	clock_lock,\n";
	print EX_FILE "\t output 	coreclk,\n";
	print EX_FILE "\t output 	usrclk,\n";
	print EX_FILE "\t output 	refclkout,\n";
	print EX_FILE "\t output 	txclkout,\n";
	print EX_FILE "\t output 	plllkdetout,\n";
	print EX_FILE "\t output 	resetdone,\n";
	print EX_FILE "\t input  [$lanes_1 : 0] RXP,\n";
	print EX_FILE "\t input  [$lanes_1 : 0] RXN,\n";
	print EX_FILE "\t output [$lanes_1 : 0] TXP,\n";
	print EX_FILE "\t output [$lanes_1 : 0] TXN,\n";
	print EX_FILE "\t output 	frame_error,\n";
	print EX_FILE "\t output 	system_error,\n";
	print EX_FILE "\t output [1:0]	l0rxmac_link_error,\n";
	print EX_FILE "\t output [7:0] error_count,\n";
	print EX_FILE "\t input 	rx_dst_dsc_n,\n";
	print EX_FILE "\t input 	rx_dst_rdy_n,\n";
	print EX_FILE "\t input 	tx_src_dsc_n\n";
	print EX_FILE "\t);\n\n";

	print EX_FILE "wire [63:0] tx_data, rx_data;\n";
	print EX_FILE "wire [2:0]	tx_rem, rx_rem;\n";
	print EX_FILE "wire 			tx_src_rdy_n, rx_src_rdy_n;\n";
	print EX_FILE "wire 			tx_dst_rdy_n;\n";
	print EX_FILE "wire 			tx_sof_n, rx_sof_n;\n";
	print EX_FILE "wire 			tx_eof_n, rx_eof_n;\n";
	print EX_FILE "wire 			tx_dst_dsc_n;\n";
	print EX_FILE "wire 			rx_src_dsc_n;\n";
	print EX_FILE "wire 			refclkout_i;\n";
	print EX_FILE "wire 			link_ready_i;\n";
	print EX_FILE "wire 			system_error_i;\n";
	print EX_FILE "wire [1:0]	l0rxmac_link_error_i;\n";
	print EX_FILE "wire [7:0]	error_count_i;\n";
	print EX_FILE "wire			usrclk_i;\n";
	print EX_FILE "wire			ll_user_clk;\n\n";
	print EX_FILE "wire [3:0]	link_width_i;\n\n";
	
	print EX_FILE "//---Chipscope Signals----//\n";	
	print EX_FILE "wire [159:0]DEBUG;\n";	
	print EX_FILE "wire [35:0] control;\n";

	print EX_FILE "\tassign ll_user_clk = usrclk_i;\n";
	print EX_FILE "\tassign usrclk = usrclk_i;\n";
	print EX_FILE "\tassign link_ready = link_ready_i;\n";
	print EX_FILE "\tassign link_width = link_width_i;\n";
	print EX_FILE "\tassign system_error = system_error_i;\n";
	print EX_FILE "\tassign l0rxmac_link_error = l0rxmac_link_error_i;\n";
	print EX_FILE "\tassign error_count = error_count_i;\n\n";

	if ($master) {
		print EX_FILE "connectivity_master test_master (\n";
	} else {
		print EX_FILE "connectivity_slave test_slave (\n";
	}
	print EX_FILE "\t.LINK_READY(link_ready_i),\n";
	print EX_FILE "\t.LINKUP(linkup),\n";
	print EX_FILE "\t.LINK_WIDTH(link_width_i),\n";
	print EX_FILE "\t.CLOCK_LOCK(clock_lock),\n";
	print EX_FILE "\t.REFCLK_N(clk_n),\n";
	print EX_FILE "\t.REFCLK_P(clk_p),\n";
	print EX_FILE "\t.CORECLK(coreclk),\n";
	print EX_FILE "\t.USERCLK(usrclk_i),\n";
	print EX_FILE "\t.REFCLKOUT(refclkout),\n";
	print EX_FILE "\t.TXCLKOUT(txclkout),\n";
	print EX_FILE "\t.RXP(RXP),\n";
	print EX_FILE "\t.RXN(RXN),\n";
	print EX_FILE "\t.TXP(TXP),\n";
	print EX_FILE "\t.TXN(TXN),\n";
	print EX_FILE "\t.PLLLKDETOUT(plllkdetout),\n";
	print EX_FILE "\t.RESETDONE(resetdone),\n";
	print EX_FILE "\t.FRAME_ERROR(frame_error),\n";
	print EX_FILE "\t.GTPRESET_N(gtpreset_n),\n";
	print EX_FILE "\t.LLUI_TX_DATA(tx_data),\n";
	print EX_FILE "\t.LLUI_TX_SRC_RDY_N(tx_src_rdy_n),\n";
	print EX_FILE "\t.LLUI_TX_DST_RDY_N(tx_dst_rdy_n),\n";
	print EX_FILE "\t.LLUI_TX_SOF_N(tx_sof_n),\n";
	print EX_FILE "\t.LLUI_TX_EOF_N(tx_eof_n),\n";
	print EX_FILE "\t.LLUI_TX_REM(tx_rem),\n";
	print EX_FILE "\t.LLUI_TX_SRC_DSC_N(tx_src_dsc_n),\n";
	print EX_FILE "\t.LLUI_TX_DST_DSC_N(tx_dst_dsc_n),\n";
	print EX_FILE "\t.LLUI_RX_DATA(rx_data),\n";
	print EX_FILE "\t.LLUI_RX_SRC_RDY_N(rx_src_rdy_n),\n";
	print EX_FILE "\t.LLUI_RX_DST_RDY_N(rx_dst_rdy_n),\n";
	print EX_FILE "\t.LLUI_RX_SOF_N(rx_sof_n),\n";
	print EX_FILE "\t.LLUI_RX_EOF_N(rx_eof_n),\n";
	print EX_FILE "\t.LLUI_RX_REM(rx_rem),\n";
	print EX_FILE "\t.LLUI_RX_SRC_DSC_N(rx_src_dsc_n),\n";
	print EX_FILE "\t.LLUI_RX_DST_DSC_N(rx_dst_dsc_n),\n";
	print EX_FILE "\t.RESET_INIT(reset_init),\n";
	print EX_FILE "\t.RESET_N(rst_n),\n";
	print EX_FILE "\t.SYSTEM_ERROR(system_error_i),\n";
	print EX_FILE "\t.L0RXMAC_LINK_ERROR(l0rxmac_link_error_i)\n";
	print EX_FILE "\t);\n\n";

	print EX_FILE "FRAME_GEN gen_frame (\n";
	print EX_FILE "\t.TX_D(tx_data),\n";
   print EX_FILE "\t.TX_REM(tx_rem),\n";
   print EX_FILE "\t.TX_SOF_N(tx_sof_n),\n";
   print EX_FILE "\t.TX_EOF_N(tx_eof_n),\n";
   print EX_FILE "\t.TX_SRC_RDY_N(tx_src_rdy_n),\n";
   print EX_FILE "\t.TX_DST_RDY_N(tx_dst_rdy_n),\n";
   print EX_FILE "\t.USER_CLK(ll_user_clk),\n";
   print EX_FILE "\t.RESET(~link_ready_i)\n";
	print EX_FILE "\t);\n\n";
	
	print EX_FILE "FRAME_CHECK check_frame (\n";
	print EX_FILE "\t.RX_D(rx_data),\n";
  	print EX_FILE "\t.RX_REM(rx_rem),\n";
  	print EX_FILE "\t.RX_SOF_N(rx_sof_n),\n";
  	print EX_FILE "\t.RX_EOF_N(rx_eof_n),\n";
  	print EX_FILE "\t.RX_SRC_RDY_N(rx_src_rdy_n),\n";
  	print EX_FILE "\t.USER_CLK(ll_user_clk),\n";
  	print EX_FILE "\t.RESET(~link_ready_i),\n";
	print EX_FILE "\t.ERROR_COUNT(error_count_i)\n";
	print EX_FILE "\t);\n\n";

	print EX_FILE "//---Signals for Chipscope---//\n";
	print EX_FILE "assign DEBUG = {\n";
	print EX_FILE "\t\tlink_width_i,\t\t//--[159:156]\n";
	print EX_FILE "\t\tlink_ready_i,\t\t//--[155]\n";
	print EX_FILE "\t\terror_count_i,\t\t//--[154:147]\n";
	print EX_FILE "\t\tframe_error,\t\t//--[146]\n";
	print EX_FILE "\t\ttx_data,\t\t\t\t//--[145:82]\n";
	print EX_FILE "\t\ttx_sof_n,\t\t\t//--[81]\n";
	print EX_FILE "\t\ttx_eof_n,\t\t\t//--[80]\n";
	print EX_FILE "\t\ttx_src_rdy_n,\t\t//--[79]\n";
	print EX_FILE "\t\ttx_dst_rdy_n,\t\t//--[78]\n";
	print EX_FILE "\t\ttx_rem,\t\t\t\t//--[77:75]\n";
	print EX_FILE "\t\ttx_dst_dsc_n,\t\t//--[74]\n";
	print EX_FILE "\t\ttx_src_dsc_n,\t\t//--[73]\n";
	print EX_FILE "\t\trx_data,\t\t\t\t//--[72:9]\n";
	print EX_FILE "\t\trx_src_rdy_n,\t\t//--[8]\n";
	print EX_FILE "\t\trx_dst_rdy_n,\t\t//--[7]\n";
	print EX_FILE "\t\trx_sof_n,\t\t\t//--[6]\n";
	print EX_FILE "\t\trx_eof_n,\t\t\t//--[5]\n";
	print EX_FILE "\t\trx_rem,\t\t\t\t//--[4:2]\n";
	print EX_FILE "\t\trx_dst_dsc_n,\t\t//--[1]\n";
	print EX_FILE "\t\trx_src_dsc_n\t\t//--[0]\n";
	print EX_FILE "\t};\n";

	print EX_FILE "//----ILA Core instance----//\n";
	print EX_FILE "ila i_ila (\n";
	print EX_FILE "\t.control(control),\n";
	print EX_FILE "\t.clk(ll_user_clk),\n";
	print EX_FILE "\t.data(DEBUG),\n";
	print EX_FILE "\t.trig0(tx_sof_n),\n";
	print EX_FILE "\t.trig1(tx_eof_n),\n";
	print EX_FILE "\t.trig2(rx_sof_n),\n";
	print EX_FILE "\t.trig3(rx_eof_n),\n";
	print EX_FILE "\t.trig4(tx_dst_rdy_n),\n";
	print EX_FILE "\t.trig5(rx_dst_rdy_n)\n";
	print EX_FILE "\t);\n";

	print EX_FILE "//----ICON Core instance----//\n";
	print EX_FILE "icon i_icon (\n";
	print EX_FILE "\t.control0(control)\n";
	print EX_FILE "\t);\n\n";

	print EX_FILE "\t//synthesis translate_off\n";
	print EX_FILE "\talways @(negedge tx_eof_n)\n";
	print EX_FILE "\t\tif (link_ready_i & !tx_dst_rdy_n)\n";
	if ($master) {
		print EX_FILE "\t\t\t\$display(\"\%d\", \$time, \": Master transmits frame\");\n";
	} else {
		print EX_FILE "\t\t\t\$display(\"\%d\", \$time, \": Slave transmits frame\");\n";
	}
	print EX_FILE "\talways @(negedge rx_eof_n)\n";
   print EX_FILE "\t\tif (link_ready_i & !rx_dst_rdy_n & !rx_src_rdy_n)\n";
   if ($master) {
      print EX_FILE "\t\t\t\$display(\"\%d\", \$time, \": Master receives frame\");\n";
   } else {
      print EX_FILE "\t\t\t\$display(\"\%d\", \$time, \": Slave receives frame\");\n";
   }
	print EX_FILE "\t//synthesis translate_on\n";

	print EX_FILE "endmodule\n\n";

	print EX_FILE "module ila (\n";
	print EX_FILE "\tinput [35:0] control,\n";
	print EX_FILE "\tinput 			clk,\n";
	print EX_FILE "\tinput [159:0] data,\n";
	print EX_FILE "\tinput [0:0]   trig0,\n";
	print EX_FILE "\tinput [0:0]	 trig1,\n";
	print EX_FILE "\tinput [0:0]   trig2,\n";
	print EX_FILE "\tinput [0:0]	 trig3,\n";
	print EX_FILE "\tinput [0:0]	 trig4,\n";
	print EX_FILE "\tinput [0:0]	 trig5\n";
	print EX_FILE "\t);\n";
	print EX_FILE "endmodule\n\n";
	
	print EX_FILE "module icon (\n";
	print EX_FILE "\toutput [35:0] control0\n";
	print EX_FILE "\t);\n";
	print EX_FILE "endmodule\n";
}


##------------Subroutine for writing TOP file------------------##
sub write_top_file
{
	my ($master,$lanes,$design_option) = @_;
	my $lanes_1	= $lanes - 1;
		##--print file header--##
	print TOP_FILE "//------------------------------------------------------------------------------\n";
   print TOP_FILE "//\n";
   print TOP_FILE "// Copyright (C) 2007, Xilinx, Inc. All Rights Reserved.\n";
   print TOP_FILE "//\n";
   print TOP_FILE "// This file is owned and controlled by Xilinx and must be used solely\n";
   print TOP_FILE "// for design, simulation, implementation and creation of design files\n";
   print TOP_FILE "// limited to Xilinx devices or technologies. Use with non-Xilinx\n";
   print TOP_FILE "// devices or technologies is expressly prohibited and immediately\n";
   print TOP_FILE "// terminates your license.\n";
   print TOP_FILE "//\n";
   print TOP_FILE "// Xilinx products are not intended for use in life support\n";
   print TOP_FILE "// appliances, devices, or systems. Use in such applications is\n";
   print TOP_FILE "// expressly prohibited.\n";
   print TOP_FILE "//\n";	
	print TOP_FILE "//   ____  ____\n";
   print TOP_FILE "//  /   /\\/   /\n";
   print TOP_FILE "// /___/  \\  /    Vendor      : Xilinx\n";
   print TOP_FILE "// \\   \\   \\/     Version     : 1.0\n";
   print TOP_FILE "//  \\   \\         Application : XAPP869\n";
   if ($master){
   print TOP_FILE "//  /   /         Filename    : connectivity_master.v\n"
   }else{
   print TOP_FILE "//  /   /         Filename    : connectivity_slave.v\n"
   }
   print TOP_FILE "// /___/   /\\\n";
   print TOP_FILE "// \\   \\  /  \\\n";
   print TOP_FILE "//  \\___\\/\\___\\\n";
   print TOP_FILE "//\n";
   print TOP_FILE "//------------------------------------------------------------------------------\n";

	print TOP_FILE "\`timescale 1 ns / 1 ps\n";
	if ($master) {	
		print TOP_FILE "module connectivity_master (\n";
	} else {
		print TOP_FILE "module connectivity_slave (\n";
	}
	print TOP_FILE "\toutput         LINK_READY,\n";
	print TOP_FILE "\toutput         LINKUP,\n";
	print TOP_FILE "\toutput [3:0]   LINK_WIDTH,\n";
	print TOP_FILE "\toutput         CLOCK_LOCK,\n";
	print TOP_FILE "\tinput          REFCLK_N,\n";
	print TOP_FILE "\tinput          REFCLK_P,\n";
	print TOP_FILE "\toutput         CORECLK,\n";
	print TOP_FILE "\toutput         USERCLK,\n";
	print TOP_FILE "\toutput         REFCLKOUT,\n";
	print TOP_FILE "\toutput         TXCLKOUT,\n";
	print TOP_FILE "\tinput  [$lanes_1 : 0] RXP,\n";
	print TOP_FILE "\tinput  [$lanes_1 : 0] RXN,\n";
	print TOP_FILE "\toutput [$lanes_1 : 0] TXN,\n";
	print TOP_FILE "\toutput [$lanes_1 : 0] TXP,\n";
	print TOP_FILE "\toutput         PLLLKDETOUT,\n";
	print TOP_FILE "\toutput         RESETDONE,\n";
	print TOP_FILE "\toutput         FRAME_ERROR,\n";
	print TOP_FILE "\tinput [63:0]   LLUI_TX_DATA,\n";
	print TOP_FILE "\tinput          LLUI_TX_SRC_RDY_N,\n";
	print TOP_FILE "\toutput         LLUI_TX_DST_RDY_N,\n";
	print TOP_FILE "\tinput          LLUI_TX_SOF_N,\n";
	print TOP_FILE "\tinput          LLUI_TX_EOF_N,\n";
	print TOP_FILE "\tinput [2:0]    LLUI_TX_REM,\n";
	print TOP_FILE "\tinput          LLUI_TX_SRC_DSC_N,\n";
	print TOP_FILE "\toutput         LLUI_TX_DST_DSC_N,\n";
	print TOP_FILE "\toutput [63:0]  LLUI_RX_DATA,\n";
	print TOP_FILE "\toutput         LLUI_RX_SRC_RDY_N,\n";
	print TOP_FILE "\tinput          LLUI_RX_DST_RDY_N,\n";
	print TOP_FILE "\toutput         LLUI_RX_SOF_N,\n";
	print TOP_FILE "\toutput         LLUI_RX_EOF_N,\n";
	print TOP_FILE "\toutput [2:0]   LLUI_RX_REM,\n";
	print TOP_FILE "\toutput         LLUI_RX_SRC_DSC_N,\n";
	print TOP_FILE "\tinput          LLUI_RX_DST_DSC_N,\n";
	print TOP_FILE "\tinput          GTPRESET_N,\n";
	print TOP_FILE "\tinput          RESET_INIT,\n";
	print TOP_FILE "\tinput          RESET_N,\n";
	print TOP_FILE "\toutput         SYSTEM_ERROR,\n";
	print TOP_FILE "\toutput [1:0]   L0RXMAC_LINK_ERROR\n";
	print TOP_FILE "\t);\n\n";
	print TOP_FILE "\treg [0:3] debounce_rst_r;\n";
	print TOP_FILE "\treg [0:3] debounce_gtpreset;\n";
	print TOP_FILE "\treg [0:3] debounce_rst_init_r;\n";
	print TOP_FILE "\twire reset_init_i;\n";
	print TOP_FILE "\twire GTPRESET_i;\n";
	print TOP_FILE "\twire RST_i;\n";
	print TOP_FILE "\twire REFCLK;\n\n";
	print TOP_FILE "\twire CORE_CLK;\n\n";
	print TOP_FILE "\twire REFCLK_OUT;\n\n";
	
	print TOP_FILE "\tassign CORECLK = CORE_CLK;\n";
	print TOP_FILE "\tassign REFCLKOUT = REFCLK_OUT;\n\n";

	print TOP_FILE "\t//--Differential to single-ended clock conversion--//\n";
	print TOP_FILE "\tIBUFDS mgtclk (.O(REFCLK), .I(REFCLK_P), .IB(REFCLK_N));\n\n";
	print TOP_FILE "\t//---User master reset--//\n";
	print TOP_FILE "\talways @(posedge CORE_CLK)\n";
	print TOP_FILE "\t\tdebounce_rst_init_r  <= {RESET_INIT,debounce_rst_init_r[0:2]};\n\n";
	print TOP_FILE "\tassign reset_init_i  =  &debounce_rst_init_r;\n\n";
	print TOP_FILE "\talways @(posedge CORE_CLK)\n";
	print TOP_FILE "\t\tif (reset_init_i)\n";
	print TOP_FILE "\t\t\tdebounce_rst_r <= 4'b1111;\n";
	print TOP_FILE "\t\telse\n";
	print TOP_FILE "\t\t\tdebounce_rst_r <=  {~RESET_N, debounce_rst_r[0:2]};\n\n";
	print TOP_FILE "\tassign  RST_i  =  &debounce_rst_r;\n\n";
	print TOP_FILE "\t//--GTP Reset--//\n";
	print TOP_FILE "\talways @(posedge REFCLK_OUT)\n";
	print TOP_FILE "\t\tdebounce_gtpreset <=  {~GTPRESET_N,debounce_gtpreset[0:2]};\n\n";
	print TOP_FILE "\tassign  GTPRESET_i  =  & debounce_gtpreset;\n\n";
	if ($master) {
		print TOP_FILE "\tconnectivity_master_top #(\n";
	} else {
		print TOP_FILE "\tconnectivity_slave_top #(\n";
	}
	print TOP_FILE "\t.NO_OF_LANES($lanes),\n";
	print TOP_FILE "\t.DESIGN_OPTION($design_option)\n";
	if ($master) {
		print TOP_FILE "\t) test_master_top (\n";
	} else {
		print TOP_FILE "\t) test_slave_top (\n";
	}
	print TOP_FILE "\t.LINK_READY(LINK_READY),\n";
	print TOP_FILE "\t.LINKUP(LINKUP),\n";
	print TOP_FILE "\t.LINK_WIDTH(LINK_WIDTH),\n";
	print TOP_FILE "\t.CLOCK_LOCK(CLOCK_LOCK),\n";
	print TOP_FILE "\t.REFCLK(REFCLK),\n";
	print TOP_FILE "\t.CORECLK(CORE_CLK),\n";
	print TOP_FILE "\t.USERCLK(USERCLK),\n";
	print TOP_FILE "\t.REFCLKOUT(REFCLK_OUT),\n";
	print TOP_FILE "\t.TXCLKOUT(TXCLKOUT),\n";
	print TOP_FILE "\t.RXN(RXN),\n";
	print TOP_FILE "\t.RXP(RXP),\n";
	print TOP_FILE "\t.TXN(TXN),\n";
	print TOP_FILE "\t.TXP(TXP),\n";
	print TOP_FILE "\t.PLLLKDETOUT(PLLLKDETOUT),\n";
	print TOP_FILE "\t.RESETDONE(RESETDONE),\n";
	print TOP_FILE "\t.FRAME_ERROR(FRAME_ERROR),\n";
	print TOP_FILE "\t.LLUI_TX_DATA(LLUI_TX_DATA),\n";
	print TOP_FILE "\t.LLUI_TX_SRC_RDY_N(LLUI_TX_SRC_RDY_N),\n";
	print TOP_FILE "\t.LLUI_TX_DST_RDY_N(LLUI_TX_DST_RDY_N),\n";
	print TOP_FILE "\t.LLUI_TX_SOF_N(LLUI_TX_SOF_N),\n";
	print TOP_FILE "\t.LLUI_TX_EOF_N(LLUI_TX_EOF_N),\n";
	print TOP_FILE "\t.LLUI_TX_REM(LLUI_TX_REM),\n";
	print TOP_FILE "\t.LLUI_TX_SRC_DSC_N(LLUI_TX_SRC_DSC_N),\n";
	print TOP_FILE "\t.LLUI_TX_DST_DSC_N(LLUI_TX_DST_DSC_N),\n";
	print TOP_FILE "\t.LLUI_RX_DATA(LLUI_RX_DATA),\n";
	print TOP_FILE "\t.LLUI_RX_SRC_RDY_N(LLUI_RX_SRC_RDY_N),\n";
	print TOP_FILE "\t.LLUI_RX_DST_RDY_N(LLUI_RX_DST_RDY_N),\n";
	print TOP_FILE "\t.LLUI_RX_SOF_N(LLUI_RX_SOF_N),\n";
	print TOP_FILE "\t.LLUI_RX_EOF_N(LLUI_RX_EOF_N),\n";
	print TOP_FILE "\t.LLUI_RX_REM(LLUI_RX_REM),\n";
	print TOP_FILE "\t.LLUI_RX_SRC_DSC_N(LLUI_RX_SRC_DSC_N),\n";
	print TOP_FILE "\t.LLUI_RX_DST_DSC_N(LLUI_RX_DST_DSC_N),\n";
	print TOP_FILE "\t.GTPRESET(GTPRESET_i),\n";
	print TOP_FILE "\t.RESET(RST_i),\n";
	print TOP_FILE "\t.SYSTEM_ERROR(SYSTEM_ERROR),\n";
	print TOP_FILE "\t.L0RXMAC_LINK_ERROR(L0RXMAC_LINK_ERROR)\n";
	print TOP_FILE "\t);\n";

	print TOP_FILE "endmodule\n";
}

##------------Subroutine for writing SCR file------------------##
sub write_scr_file
{
	my ($master,$use_example,$device) = @_;
	
	print SCR_FILE "\n";
	print SCR_FILE "run\n";
	if ($master) {
			print SCR_FILE "-ifn make_master.prj\n";
	} else {
			print SCR_FILE "-ifn make_slave.prj\n";
	}
	print SCR_FILE "-ifmt mixed\n";
	if ($master) {
		if ($use_example) {
			print SCR_FILE "-ofn connectivity_sample_master.ngc\n";
		} else {
			print SCR_FILE "-ofn connectivity_master.ngc\n";
		}
	} else {
		if ($use_example) {
			print SCR_FILE "-ofn connectivity_sample_slave.ngc\n";
		} else {
			print SCR_FILE "-ofn connectivity_slave.ngc\n";
		}			
	}
	print SCR_FILE "-ofmt NGC\n";
	print SCR_FILE "-p $device\n";
	if ($master) {
		if ($use_example) {
			print SCR_FILE "-top connectivity_sample_master\n";
		} else {
			print SCR_FILE "-top connectivity_master\n";	
		}
	} else {
		if ($use_example) {
			print SCR_FILE "-top connectivity_sample_slave\n";
		} else {
			print SCR_FILE "-top connectivity_slave\n";	
		}
	}
	print SCR_FILE "-keep_hierarchy NO\n";
	print SCR_FILE "-rtlview Yes\n";
	print SCR_FILE "-glob_opt AllClockNets\n";
	print SCR_FILE "-read_cores YES\n";
	print SCR_FILE "-write_timing_constraints NO\n";
	print SCR_FILE "-cross_clock_analysis NO\n";
	print SCR_FILE "-hierarchy_separator /\n";
	print SCR_FILE "-bus_delimiter <>\n";
	print SCR_FILE "-case maintain\n";
	print SCR_FILE "-slice_utilization_ratio 100\n";
	print SCR_FILE "-dsp_utilization_ratio 100\n";
	print SCR_FILE "-verilog2001 YES\n";
	print SCR_FILE "-vlgincdir {\"../example_design\"}\n";
	print SCR_FILE "-fsm_extract YES\n";
	print SCR_FILE "-fsm_encoding Auto\n";
	print SCR_FILE "-safe_implementation No\n";
	print SCR_FILE "-fsm_style lut\n";
	print SCR_FILE "-ram_extract Yes\n";
	print SCR_FILE "-ram_style Auto\n";
	print SCR_FILE "-rom_extract Yes\n";
	print SCR_FILE "-mux_style Auto\n";
	print SCR_FILE "-decoder_extract YES\n";
	print SCR_FILE "-priority_extract YES\n";
	print SCR_FILE "-shreg_extract YES\n";
	print SCR_FILE "-shift_extract YES\n";
	print SCR_FILE "-xor_collapse YES\n";
	print SCR_FILE "-rom_style Auto\n";
	print SCR_FILE "-mux_extract YES\n";
	print SCR_FILE "-resource_sharing YES\n";
	print SCR_FILE "-use_dsp48 auto\n";
	print SCR_FILE "-iobuf YES\n";
	print SCR_FILE "-max_fanout 100\n";
	print SCR_FILE "-bufg 32\n";
	print SCR_FILE "-register_duplication YES\n";
	print SCR_FILE "-register_balancing No\n";
	print SCR_FILE "-slice_packing YES\n";
	print SCR_FILE "-optimize_primitives NO\n";
	print SCR_FILE "-use_clock_enable Auto\n";
	print SCR_FILE "-use_sync_set Auto\n";
	print SCR_FILE "-use_sync_reset Auto\n";
	print SCR_FILE "-iob auto\n";
	print SCR_FILE "-equivalent_register_removal YES\n";
	print SCR_FILE "-slice_utilization_ratio_maxmargin 5\n";
	
}
	##------------Subroutine for writing PRJ file------------------##
sub write_prj_file
{
	my (@MODULES_LIST) = @_;
	$xilinx_path = $ENV{XILINX};
	print PRJ_FILE "verilog work $xilinx_path/virtex5/verilog/src/iSE/unisim_comp.v\n";
	foreach $module(@MODULES_LIST) {
		print PRJ_FILE "verilog work ../$module\n";
	}
	
}
