
//------------------------------------------------------------------------------
//
// Copyright (C) 2006, Xilinx, Inc. All Rights Reserved.
//
// This file is owned and controlled by Xilinx and must be used solely
// for design, simulation, implementation and creation of design files
// limited to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and immediately
// terminates your license.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor      : Xilinx
// \   \   \/     Version     : 1.0
//  \   \         Application : XAPP869
//  /   /         Filename    : connectivity_sample_tb.v
// /___/   /\     
// \   \  /  \
//  \___\/\___\
//
//------------------------------------------------------------------------------

//--Description : Sample testbench to simulate the example design.

`timescale 1 ns/1 ps

module connectivity_sample_tb ();

parameter integer NO_OF_LANES = 4;
parameter integer CORE_CLK_PERIOD = 4;
parameter integer REF_CLK_PERIOD = 10;

reg clk;
reg rst_n;
wire link_up_master, link_up_slave;
wire link_ready_master, link_ready_slave;
wire [3:0] link_width_master, link_width_slave;
wire	frame_error_master, frame_error_slave;
wire [0:7] error_cnt_master, error_cnt_slave;
wire clock_lock_master, clock_lock_slave;
wire [NO_OF_LANES-1:0] rxp,rxn,txp,txn;

//Clock
    initial
        clk = 1'b0;
    always  
        #(REF_CLK_PERIOD / 2) clk = ~clk;

	initial
	begin
		rst_n = 1'b0;
		#(1*CORE_CLK_PERIOD)

        // Wait for clock lock on master & slave
      while (clock_lock_master != 1'b1 || clock_lock_slave != 1'b1) begin
         #1;
      end

      #(10*CORE_CLK_PERIOD)
      rst_n = 1'b1;

		while (link_ready_master != 1'b1 || link_ready_slave != 1'b1)
		begin
			#1;
		end
		
		#(10*CORE_CLK_PERIOD);

	end


connectivity_sample_master DUT_master (
	.clk_n(~clk),
	.clk_p(clk),
   .rst_n(rst_n),
	.reset_init(1'b0),
	.gtpreset_n(1'b1),
   .link_ready(link_ready_master),
	.linkup(link_up_master),
   .link_width(link_width_master),
	.clock_lock(clock_lock_master),
	.coreclk(),
   .usrclk(),
   .refclkout(),
   .txclkout(),
   .plllkdetout(),
   .resetdone(),
   .RXP(rxp),
   .RXN(rxn),
   .TXP(txp),
   .TXN(txn),
   .frame_error(frame_error_master),
	.system_error(),
	.l0rxmac_link_error(),
   .error_count(error_cnt_master),
	.rx_dst_dsc_n(1'b1),
   .rx_dst_rdy_n(1'b0),
   .tx_src_dsc_n(1'b1)
);

connectivity_sample_slave DUT_slave (
	.clk_n(~clk),
	.clk_p(clk),
   .rst_n(rst_n),
	.reset_init(1'b0),
	.gtpreset_n(1'b1),
   .link_ready(link_ready_slave),
	.linkup(link_up_slave),
   .link_width(link_width_slave),
	.clock_lock(clock_lock_slave),
	.coreclk(),
   .usrclk(),
   .refclkout(),
   .txclkout(),
   .plllkdetout(),
   .resetdone(),
   .RXP(txp),
   .RXN(txn),
   .TXP(rxp),
   .TXN(rxn),
   .frame_error(frame_error_slave),
	.system_error(),
	.l0rxmac_link_error(),
   .error_count(error_cnt_slave),
	.rx_dst_dsc_n(1'b1),
   .rx_dst_rdy_n(1'b0),
   .tx_src_dsc_n(1'b1)
);

	initial begin
		wait (clock_lock_slave == 1'b1 && clock_lock_master == 1)
			$display ("===========================================");
			$display ("%d", $time, ": Clock Lock asserted at both ends");
		wait (link_up_slave == 1'b1 && link_up_master == 1'b1)
			$display ("%d", $time, ": PCI Express Link Up asserted");
		wait (link_ready_slave == 1'b1)
			$display ("--------------------------------------");
			$display ("%d", $time, ": Link Ready for data transfer");
			$display ("============================================");
	end

endmodule
